package com.bcxin.identity.api.impls;

import cn.hutool.core.util.StrUtil;
import com.bcxin.Infrastructures.TenantContext;
import com.bcxin.Infrastructures.exceptions.BadTenantException;
import com.bcxin.Infrastructures.utils.JwtUtil;
import com.bcxin.api.interfaces.ApiConstant;
import com.bcxin.api.interfaces.identities.IdentityRpcProvider;
import com.bcxin.api.interfaces.identities.requests.*;
import com.bcxin.api.interfaces.identities.responses.PrepareResetPasswordResponse;
import com.bcxin.api.interfaces.identities.responses.SignInResponse;
import com.bcxin.api.interfaces.tenants.requests.employees.ResetPassWordRequest;
import com.bcxin.identity.api.utils.RSAUtils;
import com.bcxin.identity.domains.components.PasswordEncoder;
import com.bcxin.identity.domains.components.SmsExecutor;
import com.bcxin.identity.domains.entities.IdentityUserEntity;
import com.bcxin.identity.domains.entities.UserNamePasswordPrincipalEntity;
import com.bcxin.identity.domains.enums.ForgetPasswordAction;
import com.bcxin.identity.domains.exceptions.IdentityNotFoundException;
import com.bcxin.identity.domains.repositories.IdentityUserRepository;
import com.bcxin.identity.domains.services.IdentityUserService;
import com.bcxin.identity.domains.services.commandResult.PrepareResetPasswordCommandResult;
import com.bcxin.identity.domains.services.commands.*;
import org.apache.dubbo.config.annotation.DubboService;
import org.apache.dubbo.rpc.RpcContext;

import java.util.*;
import java.util.stream.Collectors;

@DubboService(version = ApiConstant.VERSION,validation = "true",timeout = 120 *1000,retries = 0)
public class IdentityRpcProviderImpl implements IdentityRpcProvider {
    private final IdentityUserService identityUserService;
    private final IdentityUserRepository identityUserRepository;
    private final PasswordEncoder passwordEncoder;
    private final SmsExecutor smsExecutor;

    public IdentityRpcProviderImpl(IdentityUserService identityUserService,
                                   IdentityUserRepository identityUserRepository,
                                   SmsExecutor smsExecutor,PasswordEncoder passwordEncoder) {
        this.identityUserService = identityUserService;
        this.identityUserRepository = identityUserRepository;
        this.smsExecutor = smsExecutor;
        this.passwordEncoder = passwordEncoder;
    }

    @Override
    public String getByName() {
        StringBuilder sb = new StringBuilder();
        RpcContext rpcContext = RpcContext.getServerContext();

        sb.append(String.format("%s:", new Date()));
        sb.append(String.format("URL=%s;", rpcContext.getConsumerUrl()));
        sb.append(String.format("注册信息:%s:%s:%s;", rpcContext.getProtocol(), rpcContext.getRemoteHost(), rpcContext.getRemotePort()));
        if (rpcContext.getArguments() != null) {
            Arrays.stream(rpcContext.getArguments()).forEach(arg -> {
                sb.append(arg);
            });
        }

        return sb.toString();
    }

    @Override
    public void changePassword(ChangePasswordRequest request) {
        ChangePasswordCommand command =
                ChangePasswordCommand.create(
                        request.getIdentityUserId(),
                        request.getUserName(),
                        formatFrontPassword(request.isRsa(), request.getPassword()),
                        formatFrontPassword(request.isRsa(), request.getNewPassword()),
                        formatFrontPassword(request.isRsa(), request.getConfirmedPassword())
                );

        this.identityUserService.changePassword(command);
    }
    @Override
    public void resetPassword(ResetPassWordRequest request){
        this.identityUserService.dispatch(ResetPasswordCommandAdmin.create(request.getLoginName(),request.getNewPassword()));
    }

    @Override
    public void bindWechat(BindWechatRequest request) {
        String tenantId = TenantContext.getInstance().getUserContext().get().getId();
        this.identityUserService.dispatch(WechatBindCommand.create(
                tenantId, request.getOpenId(), request.getNicky(), request.getUnionId()));
    }

    @Override
    public SignInResponse signIn(WeChatSignInRequest request) {
        WechatSignInCommand command = WechatSignInCommand.create(request.getOpenId(), request.getIpAddress());
        SignInCommand.SignInCommandResult result = this.identityUserService.dispatch(command);

        return SignInResponse.create(result.getToken(), result.getName(), result.getIdNum());
    }

    @Override
    public SignInResponse signIn(WeChatMiniSignInRequest request) {
        WechatMiniSignInCommand command = WechatMiniSignInCommand.create(request.getAppid(), request.getCode(), request.getIpAddress());
        SignInCommand.SignInCommandResult result = this.identityUserService.dispatch(command);

        return SignInResponse.create(result.getToken(), result.getName(), result.getIdNum());
    }

    @Override
    public void change(ChangeBasicIdentityRequest request) {
        this.identityUserService.dispatch(UpdateIdentityBasicCommand.create(
                request.getTenantUserId(),
                request.getName(),
                request.getTelephone(),
                request.getIdNum()));
    }

    @Override
    public void unbindWechat(String id) {
        this.identityUserService.dispatch(WechatUnBindCommand.create(id));
    }

    /**
     * description：判断是否绑定微信unionid
     * author：linchunpeng
     * date：2023/10/12
     */
    @Override
    public boolean isBindUnionId(String id) {
        return this.identityUserService.dispatch(WechatCheckUnionIdCommand.create(id));
    }

    @Override
    public PrepareResetPasswordResponse prepareResetPassword(PrepareResetPasswordRequest request) {
        ForgetPasswordAction action = null;
        if (request.getActionType() != null) {
            switch (request.getActionType()) {
                case FaceIdentity:
                    action = ForgetPasswordAction.FaceIdentity;
                    break;
                case SMS:
                    action = ForgetPasswordAction.SMS;
                    break;
            }
        }

        PrepareResetPasswordCommandResult result =
                this.identityUserService.dispatch(
                        PrepareResetPasswordCommand.create(
                                request.getLoginName(),
                                request.getData(),
                                action
                        )
                );

        if (result == null) {
            return null;
        }

        return PrepareResetPasswordResponse.create(result.getJwt(), result.isSuccess(), result.getCode(),result.getMobile());
    }

    @Override
    public PrepareResetPasswordResponse prepareResetPasswordExist(PrepareResetPasswordRequest request) {

        PrepareResetPasswordCommandResult result =
                this.identityUserService.getUserMobile(
                        PrepareResetPasswordCommand.create(
                                request.getLoginName(),
                                request.getData(),
                                null
                        )
                );

        return PrepareResetPasswordResponse.create(result.getJwt(), result.isSuccess(), result.getCode(),result.getMobile(),result.isHasHeadPhoto());
    }

    @Override
    public String unlock(UnlockRequest request) {

        if (StrUtil.isEmpty(request.getUserid())||StrUtil.isEmpty(request.getPassword())) {
            throw new BadTenantException("参数不能为空");
        }

        IdentityUserEntity identityUserEntity = this.identityUserRepository.getByTenantUserId(request.getUserid());

        if (identityUserEntity == null) {
            throw new IdentityNotFoundException();
        }
        //对比密码
        Optional<UserNamePasswordPrincipalEntity> userNamePasswordOptional =
                identityUserEntity.getPrincipals().stream().filter(ii -> ii instanceof UserNamePasswordPrincipalEntity)
                        .map(ii -> (UserNamePasswordPrincipalEntity) ii)
                        .findFirst();
        if (userNamePasswordOptional.isPresent()) {
            try {
                String password = formatFrontPassword(true, request.getPassword());
                if(passwordEncoder.isMatched(userNamePasswordOptional.get().getPassword(),password)){
                    return JwtUtil.getToken(request.getUserid());
                }else {
                    throw new BadTenantException("密码错误！");
                }

            }catch (Exception e) {
                throw new BadTenantException("密码错误！");
            }

        }
        throw new IdentityNotFoundException();
    }

    @Override
    public void prepareResetPasswordCheckCode(ResetPasswordRequest request) {
        ResetPasswordCommand command = ResetPasswordCommand.create(
                request.getJwt(), request.getLoginName(),
                null,
                request.getCode()
        );

        this.identityUserService.resetPasswordCheckCode(command);
    }
    @Override
    public void restPassword(ResetPasswordRequest request) {
        ResetPasswordCommand command = ResetPasswordCommand.create(
                request.getJwt(), request.getLoginName(),
                formatFrontPassword(request.isRsa(), request.getNewPassword()),
                request.getCode()
        );

        this.identityUserService.dispatch(command);
    }

    @Override
    public void restUsername(ResetUserNameRequest request) {
        this.identityUserService.changeUserName(ChangeUserNameCommand.create(request.getUserid(),request.getLoginName()));
    }

    @Override
    public void sendCode(SmsRequest request) {
        smsExecutor.sendCode(request);
    }

    @Override
    public boolean checkCode(SmsRequest request) {
        return smsExecutor.checkCode(request);
    }

    @Override
    public SignInResponse ssoLogin(SsoLoginRequest request) throws Exception {
        SignInCommand command = SignInCommand.create(request.getUserName(),null,false,false, request.getIpAddress());
        SignInCommand.SignInCommandResult result = this.identityUserService.signIn(command);
        return SignInResponse.create(result.getToken(), result.getName(), result.getIdNum());
    }

    @Override
    public SignInResponse ssoLoginByIdNum(String userName, String ipAddress) throws Exception {
        SignInCommand command = SignInCommand.create(userName,null,false,false, ipAddress);
        SignInCommand.SignInCommandResult result = this.identityUserService.signIn(command);
        return SignInResponse.create(result.getToken(), result.getName(), result.getIdNum());
    }

    @Override
    public SignInResponse signIn(SignInRequest request) {
        SignInCommand command = SignInCommand.create(
                request.getUserName(),
                formatFrontPassword(request.isRsa(), request.getPassword()),
                request.isFromMobile(), request.getIpAddress());

        SignInCommand.SignInCommandResult result = this.identityUserService.signIn(command);

        return SignInResponse.create(result.getToken(), result.getName(), result.getIdNum());
    }

    @Override
    public void signUp(SignUpRequest request) {
        Collection<SignUpCommand.CredentialCommandItem> credentialCommandItems = new ArrayList<>();
        if(request.getCredentials()!=null) {
            credentialCommandItems = request.getCredentials().stream().map(ii -> SignUpCommand.CredentialCommandItem.create(ii.getCredentialType(), ii.getNumber()))
                    .collect(Collectors.toList());
        }
        SignUpCommand command = SignUpCommand.create(request.getTenantUserId(), request.getTelephone(),request.getIdNum(), request.getName(),credentialCommandItems);

        this.identityUserService.signUp(command);
    }


    private static String formatFrontPassword(boolean isRsa, String password) {
        try {
            if (isRsa) {
                return RSAUtils.decrypt(password);
            }
        } catch (Exception ex) {
            System.err.println(String.format("isRsa=%s;password=%s", isRsa, password));
            ex.printStackTrace();
        }

        return password;
    }
}
